;--------------------------------------------------------------------------------------
;Hi there, this is a contribution to the 'First unofficial CSDB UOC coding challenge'.
;I did not try to use UOCs everywhere but focussed on creating two main effects in the
;upper and lower borders that apply the rarely used opcodes RLA and LAS.
;
;You won't find any RLA in this listing though, because the unrolled code for the lower border
;that contains this UOC is generated in the subroutine generateunrolledcode.
;Let me explain how RLA is used. A sprite scroller is created that moves text over some fixed
;sprite graphics. I was first considering SLO and SRE because the OR and EOR operations performed
;by them seemed more suitable than the AND operation of RLA. But for a scroller you need the carry
;for the next sprite in line. And it turns out that the AND operation works as well.
;Suppose the data of the sliding text are stored from $2400-$25ff and the data of the fixed background
;from $2200-$23ff. The actual sprite pointers point to $2000-$21ff.
;Combining the sliding and fixed data without RLA would go something like (for the rightmost byte of
;the top line of sprite 1)
;
; rol $2442
; lda $2442
; and $2242  <------ here we combine the sliding and fixed data
; sta $2042  <------ this will be displayed
;
;This takes 18 cycles and 16 bytes, but with RLA you can do
;
; lda $2242
; rla $2442  <------ rol+and in one instruction
; sta $2042  
;
;This takes only 14 cycles and 12 bytes.
;
;I used multicolor sprites and this causes changes in colors. For suitably chosen colors you can get
;a nice color mixing/interlacing effect. The layered scrolling technique also works with hires sprites.
;What is interesting is that if you pause the scroller with P, the text may not be readible, but it is
;when moving.
;
;LAS is used in the upper border raster effect. It can be the case that the stack is not used in a main routine,
;since it is cheaper to store things in the zeropage. Of course when a subroutine is called or an interrupt triggers
;the return address (and status register in case of an IRQ) is stored on the stack, but after returning to the main loop
;the stackpointer (SP) is back to the same value again. This means that you can change the SP at will in the main loop
;without messing things up. For example, you can use it as temporary storage of the X register with TXS/TSX.
;This opens up possibilities for UOCs like LAS (and TAS).
;For example, suppose I want to cycle through a list of numbers (e.g., colors for rasterbars, sprite x- and y- positions , ...)
;If the list has a length that is a power of two, you could do that with LAS. For example for 16 numbers:
;Suppose we start with SP=$f7 (any value will work).
;
; las Mask,y    if Mask is one page filled with $0f, this brings the SP to $07 (and A and X as well) for any Y.
; dex      X-->$06 
; lda Table,x
;Do something with accu
;..........
; txs           SP is now $06, so the next time 'lda List,x' will pick the next value, etc.
;
;The SP (and X) after the LAS instruction will always remain in the range 0-$0f, no need to check for that!
;Instead of the lda Table,x one could use pla if the data is on the stack and no interrupt can take place during this code snippet.
;Then the dex should be replaced by e.g. sbx #$11 to bring the SP to a safe area, to ensure the data on the stack is not messed up 
;in other parts of the code.
;
;You could also select several different list, by using a different mask according to Y.
;For example: If Mask is a page filled with for example $0f, $1f, $2f, $3f, $0f, $1f, $2f $3f, ... then you can select between
;four different lists. But then the dex / txs must be modified to ensure you get the SP back to $fx again.
;A variant of this is executed in the topborder:
;
;        ldx $d021
;        and #$0f
;        lda colorscheme,x   <------ This grabs the low byte for the desired list of 16 colors
;        sta index+1
;        ldy #startvalue
;loop    las Mask,y   <----  The mask is always $0f.
;index   lda colorcycling,x   <------ Take the next color from the selected list
;        ..........
;change colors (and waste cycles to sync with the raster)
;        ..........
;        dex
;        txs    <---- Prepare the stackpointer
;        iny
;        cpy #endvalue
;        bne loop
;
;Since this loop goes through 33=2*16+1 rasterlines, you can see that in the next frame the next color is started with.
;That is why the color bars scroll upwards (automatically).
;
;Btw, I also used ISC in the generateunrolledcode subroutine.
;
;If you have any questions, remarks, criticism, or suggestions, I will be happy to hear about them.
;For example, I don't know how it looks on real hardware.
;
;-- Color Bar.

*=$801

        BYTE    $0B, $08, $0A, $00, $9E, $32, $30, $36, $34, $00, $00, $00

*=$0810

scrcol=00       ;default colors
borcol=00
deltax=48       ;horizontal spacing of sprites in lower border
deltax2=52      ;sprite spacing in top border
;$02-$07 are first used for putting text on screen and in generateunrolledcode, so initialize AFTER the call to this routine!
scrollpos=$02   ;contains position in scrolltext
bitcounter=$03  ;how many bits of sprite 0 left to shift in
Music=$04       ;flag that tells to play music or not
resetmusic=$05  ;tells to reset the player when music is turned back on
scrollcharaddr=$06  ;address of sprite data for current character in scroller (2 bytes)
skipscrolling=$08   ;flag that tells to skip scrolling one frame or not
lastkey=$09     ;to check if a key is still pressed (or still not pressed)
lastkey2=$0a
lastkey3=$0b
lastkey4=$0c
lastkey5=$0d
Fpressed=$0e    ;flag that tells this key has been pressed
Spressed=$0f
F1pressed=$10
F3pressed=$11
F5pressed=$12
F7pressed=$13
gfxinverted=$14   ;set if F or S is pressed, then scrolling is skipped one frame to save rastertime
sgfxinverted=$15   ;toggles when S is pressed, to invert or not when refreshing scroller sprite data
colordata=$16   ;pointer to colors in top border (2 byte address)
birdindex=$18   ;indicates current bird image 0-9
birddelay=$19  ;counter to delay the updating of the bird image
birdaddr=$1a    ;offset for current birdimage w.r.t. that of image0 (i.e. 64 times the birdindex; 2 bytes)
midborcolor=$1c ;current color of middle border
Paused=$1d      ;indicates the state of the lower border scroller
scrolltextaddr=$1e      ;2 byte address
colorbardelay=$20       ;used to slowdown the upwards movement of color bars
savecolors=$21 ;to save the current bottom sprite 0-2 colors (3 bytes)
linecounter=$24 ;used in generateunrolledcode to keep track current sprite line
spritecounter=$25       ;and current sprite
Senoritas=$5900 ;need to define this since CBM Program Studio can't compile las label,y
        
;put text on screen
        lda #$00
        sta $02
        sta $04
        sta $06
        lda #$04
        sta $03
        lda #>screentext
        sta $05
        lda #$d8
        sta $07
        ldx #$04
        ldy #$00
fill    lda ($04),y
        sta ($02),y
        lda #$05        ;green
        sta ($06),y
        dey
        bne fill
        inc $03
        inc $05
        inc $07
        dex
        bne fill

        sei
        lda #$7f   ;This code was found on codebase64.org.
        sta $dc0d  ;disable timer interrupts which can be generated by the two CIA chips.
        sta $dd0d  ;stop interrupt that flashes the cursor and scans the keyboard.
        lda $dc0d  ;by reading this two registers we negate any pending CIA irqs.
        lda $dd0d  ;if we don't do this, a pending CIA irq might occur after we finish setting up our irq.
        lda #$35   ;we turn off the BASIC and KERNAL rom here.
        sta $01    ;the cpu now sees RAM everywhere except at $d000-$e000, where still the registers of SID/VICII/etc are visible
        lda #$01   ;tell the VICII to allow only raster interrupts
        sta $d01a

;------- Prepare for lower border IRQ
        lda #$ff   ;clear the interrupt condition of the VICII.
        sta $d019
        lda #$f8   ;tell at which rasterline we want the irq to be triggered
        sta $d012       ; IRQ2 opent de middenborders

        lda #$1b   ;as there are more than 256 rasterlines, the topmost bit of $d011 serves as
        sta $d011  ;the 8th bit for the rasterline we want our irq to be triggered.
        lda #$35   ;we turn off the BASIC and KERNAL rom here.
        sta $01    ;the cpu now sees RAM everywhere except at $d000-$e000, where still the registers of SID/VICII/etc are visible
        lda #<irq2  ;this is how we set up
        sta $fffe  ;the address of our interrupt code
        lda #>irq2
        sta $ffff
;initialize music player
        lda #$00
        jsr $3110       ;initialize player
        jsr generateunrolledcode        ;generate sprite scroller code 
;       initialize some variables
        lda #<scrolltext
        sta scrolltextaddr
        lda #>scrolltext
        sta scrolltextaddr+1
        lda #$01
        sta gfxinverted
        sta sgfxinverted
        sta skipscrolling
        lda #$00
        sta Fpressed
        sta Spressed
        sta Paused
        sta F1pressed
        sta F3pressed
        sta F5pressed
        sta F7pressed
        
        lda #$ff
        sta lastkey
        sta lastkey2
        sta lastkey3
        sta lastkey4
        sta lastkey5
        
        lda #<colorcycling
        sta colordata
        lda #>colorcycling
        sta colordata+1
        lda #$00
        sta birdindex
        sta birddelay
        sta colorbardelay
        sta midborcolor

        ldy #$3f
wait    lda #$00        ;space for sprite 0 and 1
        ldx gfxinverted
        beq *+4
        eor #$ff
        sta $2200,y
        sta $2240,y
        lda ImageA,y
        ldx gfxinverted
        beq *+4
        eor #$ff
        sta $2280,y
        sta $2340,y
        lda ImageL,y
        ldx gfxinverted
        beq *+4
        eor #$ff
        sta $22c0,y
        lda ImageR,y
        ldx gfxinverted
        beq *+4
        eor #$ff
        sta $2300,y
        lda ImageC,y
        ldx gfxinverted
        beq *+4
        eor #$ff
        sta $2380,y
        dey
        bpl wait

        lda #$00
        ldx sgfxinverted
        beq *+4
        eor #$ff
        ldy #$00
wait2
        sta $2400,y
        sta $2500,y
        dey
        bne wait2
        

        ldy #$3f
copybird
        lda birddata,y
        ldx gfxinverted
        beq *+4
        eor #$ff
        sta $23c0,y
        dey
        bpl copybird

        ;turn sprites on, expand and place on screen
        
        lda #$ff
        sta $d01c       ;MC 
        sta $d01d       ;x expand
        sta $d017       ;y expand
        lda #$02;0c       ;MC1
        sta $d025
        lda #$0f       ;MC2
        sta $d026
        lda #borcol
        sta $d020
        lda #scrcol
        sta $d021
        ldx #$07
        lda #$08;03
colors  sta $d027,x
        dex
        bpl colors
        lda #$10        ;x-position first sprite
        ldy #$fc        ;y-positions
        sta $d00e
        sty $d00f
        ldx #07        ;msb x pos
        stx $d010
        clc
        adc #deltax
        sta $d00c
        sty $d00d
        adc #deltax
        sta $d00a
        sty $d00b
        adc #deltax
        sta $d008
        sty $d009
        adc #deltax
        sta $d006
        sty $d007
        adc #deltax
        sta $d004
        sty $d005
        clc             ;sprite x position>255. 
        adc #deltax
        sta $d002
        sty $d003
        adc #deltax
        sta $d000
        sty $d001

        ldx #$80 ;sprite image pointers --> from $2000-$21ff
        stx $07f8
        inx
        stx $07f9
        inx
        stx $07fa
        inx
        stx $07fb
        inx
        stx $07fc
        inx
        stx $07fd
        inx
        stx $07fe
        inx
        stx $07ff
        lda #$fe
        sta $d015       ; turn sprites 1-7 on
        
        lda #$00
        sta scrollpos
        sta Music
        sta resetmusic
        lda #24
        sta bitcounter
        cli
        
        ;Code for the top border
lascode lda $d011
        bmi lascode
        lda $d012
        cmp #$0b        ;8 lines before the sprites
        bne lascode
        lda $d011       ;it sometimes happened that irq2 returned on line $10b, while it was triggered
        bmi lascode+5   ;on a line < $100. So we have to check $d011 again.
        ;wait for irq3 to trigger
        inc $cfff
        inc $cfff
        inc $cfff
        inc $cfff
nops    nop     ;irq3 will trigger during a nop operation => max 1 cycle jitter
        nop     ;which can be absorbed with an inc abs when the sprites are on
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        jmp nops ;this jump should never happen, because irq3 is made to return to onwego.

onwego  cli
        lda midborcolor
        and #$0f
        tay
        lda colorscheme,y
        sta laslus+4
        ;inc $cfff
        lda #$3f        ;sprites 0-5
        sta $d015       ; turn on
        inc $cfff
        nop
        bit $ea
        nop
        bit $ea

;combined loop with and without sprites
        ldy #$fa ;the first and last 6 lines there are no sprites and we have to waste more cycles.
;This is what cpy #21 below takes care of.
laslus
        las Senoritas,y   ; <==== This is what half of the demo is about :-D     
        lda colorcycling,x
        
        cpy #21         ;this makes that we waste extra cycles only for the raster lines without sprites
        bcc *+9
        nop
        inc $cfff       ;add 14 cycli (total 63 since bcc is not taken)        
        inc $cfff
        sta $cfff
        ;slanted P
        inc $d00a       ;should be here at cycle 50 or 51 to absorb the 1 cycle jitter
        sta $d020
        sta $d029
        sta $d028
        sta $d027
       
        dex
        txs
        iny
        cpy #27         ;33 lines in total since we started with ldy #$fa
        bne laslus     ;50 cycli when branching (sprites 0-5 on)
        
        ;wait a while to change border and screen color
        inc $cfff
        nop
        inc $cfff
        inc $cfff
        inc $cfff
        lda #borcol
        sta $d020
        lda #scrcol
        sta $d021
        

        ;slow down (move the color bands upwards every second frame)
        inc colorbardelay
        lda colorbardelay   
        and #$02
        beq @checkM     ;"Cheap" label. I should use those much more often.
        inx
        txs
        

        ;perform actions when key pressed
@checkM 
        lda resetmusic
        beq checkF
        lda #$00
        sta resetmusic
        jsr $3110       ;initialize player    
        
checkF  lda Fpressed
        beq checkS
;invert background graphics
        
        lda #$01        ;set flag to skip moving sprites
        sta skipscrolling
        eor gfxinverted
        sta gfxinverted
        ldy #$00        
        sty Fpressed   
        lda Paused
        beq invstatic  ;no need to update displayed memory, as this is done in the scroller code 
        ; when paused, the scroller code does not update the displayed memory, so we do that here
        ;sprite 0 is not displayed, but it is inverted anyway to keep the code simple
        
invstat2 lda $2200,y    
        eor #$ff
        sta $2200,y
        and $2400,y
        sta $2000,y     
        lda $2300,y
        eor #$ff
        sta $2300,y
        and $2500,y
        sta $2100,y
        iny
        bne invstat2     
        jmp checkP

invstatic lda $2200,y
        eor #$ff
        sta $2200,y
        lda $2300,y
        eor #$ff
        sta $2300,y
        iny
        bne invstatic
        jmp scrollspr
        
checkS  lda Spressed
        beq checkF1
        lda #$01
        sta skipscrolling
        eor sgfxinverted ;toggle
        sta sgfxinverted
        ldy #$00        
        sty Spressed   
        lda Paused
        beq invmoving
        
invmov2 lda $2400,y
        eor #$ff
        sta $2400,y
        and $2200,y
        sta $2000,y     
        lda $2500,y
        eor #$ff
        sta $2500,y
        and $2300,y
        sta $2100,y
        iny
        bne invmov2
        jmp checkP

invmoving lda $2400,y
        eor #$ff
        sta $2400,y
        lda $2500,y
        eor #$ff
        sta $2500,y
        iny
        bne invmoving
        jmp scrollspr
        
checkF1
        lda F1pressed
        beq checkF7
        inc midborcolor
        lda #$00
        sta F1pressed
        jmp checkP
checkF7
        lda F7pressed
        beq checkF5
        inc $d025
        lda #$00
        sta F7pressed
        jmp checkP
checkF5
        lda F5pressed
        beq checkF3
        inc $d026
        lda #$00
        sta F5pressed
        jmp checkP
checkF3
        lda F3pressed
        beq checkP
        ldy savecolors  
        iny
        sty savecolors
        sty $d027
        sty savecolors+1
        sty $d028
        sty savecolors+2
        sty $d029
        sty $d02a
        sty $d02b
        sty $d02c
        sty $d02d
        sty $d02e
        lda #$00
        sta F3pressed
checkP  
        lda Paused
        beq scrollspr
        
        lda #$01        ;set flag to skip moving sprites
        sta skipscrolling
        jmp rest
        ;scroll sprites etc
scrollspr
        lda skipscrolling
        bne rest ;don't move sprite data after inverting graphics to save rastertime
        jsr movesprites 
        dec bitcounter
        bne rest
        lda #24
        sta bitcounter
        inc scrollpos
        ldy scrollpos
        bne nextchar
        inc scrolltextaddr+1
nextchar lda (scrolltextaddr),y       ;address should be updated after 256 characters!!
        bne *+14
        ldx #$00
        stx scrollpos
        lda #>scrolltext
        sta scrolltextaddr+1
        ldy #$00
        lda (scrolltextaddr),y
        jsr decodetext

rest    lda #$00
        ora Paused
        sta skipscrolling
        
        ;place bottom border sprites
        jsr placebottomsprites

updatebird
        lda Paused
        bne halt
        inc birddelay
        ldy birddelay
        cpy #$04
        bcs *+5
        jmp halt
        ldy #$00
        sty birddelay
        inc birdindex
        ldy birdindex
        cpy #$09
        bcc *+6
        ldy #$00
        sty birdindex
        lda #$00
        sta birdaddr
        sta birdaddr+1
;calculate spritepointer
        clc
        lda birdindex
;multiply by 64
        lsr
        ror birdaddr
        lsr
        ror birdaddr
        sta birdaddr+1
;add base address sprite data
        lda #<birddata
        adc birdaddr
        sta birdaddr
        lda #>birddata
        adc birdaddr+1
        sta birdaddr+1
;copy the sprite data
        ldy #$3f
copynewbird
        lda (birdaddr),y
        ldx gfxinverted
        beq *+4
        eor #$ff
        sta $23c0,y
        dey
        bpl copynewbird

halt    jmp lascode

irq2 ; open middle borders
        pha        
        txa
        pha        
        tya
        pha        
        lda #borcol
        sta $d020
        lda #$ff   ;clear the interrupt condition of the VICII.
        sta $d019
        lda #$13        ; 24 rows to open lower/upper borders+set up MSB IRQ1 position
        sta $d011
        nop     ; wait until in the side border to change screen color
        nop
        inc $cfff
        lda midborcolor
        sta $d021       
        
;read keyboard
;All actions on colors and data are performed in main routine after top border
readkeyboard
        lda #$00
        sta $dc03       ; port b ddr (input)

keyF    lda #$04;02
        sta $dc02
        lda #$fb;fd
        sta $dc00
        lda $dc01
        cmp lastkey3
        beq readkey     ;still (not) pressed?
        sta lastkey3
        eor #$ff
        and #$20;10
        
        sta Fpressed    ;unequal to zero when pressed
        beq readkey
        jmp checkM

readkey lda #$01        ;select row
        sta $dc02       ; port a ddr (output)                       
        lda #$fe      ;select row (bit uit)
        sta $dc00       ; port a
        lda $dc01       ; port b; read keyboard
        cmp lastkey     ; don't repeat action when key is still pressed.
        
        beq KeyS
        sta lastkey
        eor #$ff
        tax
        ldy Fpressed    
        beq *+5
        jmp checkM

        
Fkey1   and #$10        ;F1. 
        sta F1pressed
        beq Fkey3    
        jmp checkM

Fkey3   txa
        and #$20        ;F3
        sta F3pressed
        beq Fkey5
        jmp checkM
Fkey5   txa
        and #$40        ;F5
        sta F5pressed
        beq Fkey7
        jmp checkM

Fkey7   txa
        and #$08        ;F7
        sta F7pressed
        beq KeyS
        jmp checkM

KeyS    lda #$02
        sta $dc02
        lda #$fd
        sta $dc00
        lda $dc01
        cmp lastkey2
        beq keyP
        sta lastkey2
        eor #$ff
        and #$20
        sta Spressed
        beq keyP
        jmp checkM
keyP    lda #$20
        sta $dc02
        lda #$df
        sta $dc00
        lda $dc01
        cmp lastkey4
        beq keyM
        sta lastkey4
        eor #$ff
        and #$02
        ;toggle scroller pausing
        eor Paused
        sta Paused
        jmp checkM
keyM    lda #$10
        sta $dc02
        lda #$ef
        sta $dc00
        lda $dc01
        cmp lastkey5
        beq checkM
        sta lastkey5
        eor #$ff
        and #$10
        ;toggle music off/on
        eor Music
        sta Music
        beq turnon
turnoff lda #$00 ;set volume to zero
        sta $d418
        jmp checkline
turnon  lda #$01        ; When the music is turned back on, I will re-initialize the player since
        sta resetmusic  ; I don't know how to just continue without artifacts (I know too little about SID).
checkM  lda music
        bne checkline
        ;dec $d020
        lda #$00
        jsr $3490       ;music player    
        ;inc $d020
        ;-------- Wait long enought to return to 25 rows
checkline 
        lda $d012
        bmi checkline
        ; ----------- set border color
        lda #borcol
        sta $d020
        lda #$93
        sta $d011       
        lda #$30   ;tell at which rasterline we want the 1st irq to be triggered (bottom of the screen)
        sta $d012
        lda #<irq1  ;this is how we set up
        sta $fffe  ;the address of our interrupt code
        lda #>irq1
        sta $ffff
        pla
        tay        
        pla
        tax        
        pla        
        rti 

;This is set to trigger when the main routine is in the middle of a bunch of nops.
;This reduces the uncertainty in the raster position to 1 cycle similar to the double interrupt method
;of syncing. The topborder code can handle 1 cycle uncertainty.
irq3    lda #$ff   ;clear the interrupt condition of the VICII.
        sta $d019
        lda #$1b
        sta $d011
        lda #$fa   ;tell at which rasterline we want the 2nd irq to be triggered
        sta $d012
        lda #<irq2  
        sta $fffe  
        lda #>irq2
        sta $ffff
        pla     ; we pull status register and return address
        pla     ; because we want the stackpointer not to change during the main routine.
        pla
        jmp onwego      ;and continue at this point in the main code (after the nops) to handle the top border.

;place top border sprites
irq1    pha        
        txa
        pha        
        tya
        pha        
        lda #borcol
        sta $d020
        ;dec $d021
        jsr placetopsprites
        ;inc $d021
    

        lda #$ff   ;clear the interrupt condition of the VICII.
        sta $d019
        lda #0
        sta $3fff
        lda #$1b        ; 25 rows
        sta $d011
        lda #$0c   ;tell at which rasterline we want the 3rd irq to be triggered
        sta $d012
        lda #<irq3  
        sta $fffe  
        lda #>irq3
        sta $ffff
        pla
        tay        
        pla
        tax        
        pla        
        rti 

placetopsprites
        ;turn sprites off, expand and place on screen
        lda #$00;$3f        ;sprites 0-5 are turned back on in top border routine
        sta $d015       ; turn off
        
        lda #$00
        sta $d017       ;y expand
        
        lda #borcol
        sta $d020
        
colors2 ldx #$02
        lda $d027,x
        sta savecolors,x   ;since these must be restored in lower border  
        dex
        bpl colors2+2
        lda #$1e        ;offset x-position sprites
        ;sta $d00a
        ldy #$13   ;y-positions
        sty $d00b
        ldx #01        ;msb x pos
        stx $d010
        clc
        adc #deltax2
        sta $d008
        sty $d009
        adc #deltax2
        sta $d006
        sty $d007
        adc #deltax2
        sta $d004
        sty $d005
        adc #deltax2
        sta $d002
        sty $d003
        clc
        adc #deltax2
        sta $d000
        sty $d001

        lda #$13        ;move letter P a bit to the left, because it will be slanted.
        sta $d00a
        

        ldx #63 ;sprite image pointers S
        stx $07f8
        ldx #58 ; A
        stx $07f9
        stx $07fc
        ldx #60 ; L
        stx $07fa
        stx $07fb
        ldx #61 ; P
        stx $07fd
        rts

placebottomsprites
        ldx #$07 ;sprite image pointers --> from $2000-$21ff
sips    txa
        ora #$80
        sta $07f8,x
        dex
        bpl sips
        ;turn sprites on, expand and place on screen
        lda #$fe
        sta $d015       ; turn on
        lda #$ff
        sta $d01c       ;MC 
        lda #$ff
        sta $d01d       ;x expand
        sta $d017       ;y expand
        lda #borcol
        sta $d020
        ldx #$02
colors3 lda savecolors,x
        sta $d027,x
        dex
        bpl colors3
        lda #$16        ;x-position first sprite
        ldy #$fc        ;y-positions
        sta $d00e
        sty $d00f
        ldx #07        ;msb x pos
        stx $d010
        clc
        adc #deltax
        sta $d00c
        sty $d00d
        adc #deltax
        sta $d00a
        sty $d00b
        adc #deltax
        sta $d008
        sty $d009
        adc #deltax
        sta $d006
        sty $d007
        adc #deltax
        sta $d004
        sty $d005
        clc             ;sprite x position>255.
        adc #deltax
        sta $d002
        sty $d003
        adc #deltax
        sta $d000
        sty $d001

        rts



*=$0e80 ;images A C L P R S (spites 0-5)
imageA  ;pointer #58
 BYTE $00,$0C,$00,$00,$39,$00,$00,$EA,$40,$00,$EA,$40,$03,$EA,$40,$03
 BYTE $AA,$90,$03,$AA,$90,$03,$AA,$90,$03,$AA,$90,$0E,$A6,$90,$0E,$93
 BYTE $A4,$0E,$93,$A4,$0E,$93,$A4,$0E,$93,$A4,$3A,$93,$A4,$3A,$AF,$A9
 BYTE $3A,$AA,$A9,$3A,$AA,$A9,$3A,$55,$E9,$3A,$40,$E9,$35,$40,$d5,$00

imageC ;pointer #59 ($0ec0) 
 BYTE $0F,$FF,$F0,$0E,$AA,$A4,$3A,$AA,$A9,$3A,$AA,$A9,$3A,$95,$69,$3A
 BYTE $40,$39,$3A,$40,$39,$3A,$40,$25,$3A,$40,$00,$3A,$40,$00,$3A,$40
 BYTE $00,$3A,$40,$00,$3A,$40,$00,$3A,$40,$3E,$3A,$40,$39,$3A,$40,$39
 BYTE $3A,$BF,$F9,$0E,$AA,$A9,$0E,$AA,$A9,$03,$AA,$A4,$00,$55,$50,$00
imageL ;pointer #60 ($0d00) 
 BYTE $3F,$c0,$00,$3A,$40,$00,$3A,$40,$00,$3A,$40,$00,$3A,$40,$00,$3A
 BYTE $40,$00,$3A,$40,$00,$3A,$40,$00,$3A,$40,$00,$3A,$40,$00,$3A,$40
 BYTE $00,$3A,$40,$00,$3A,$40,$00,$3A,$40,$0d,$3A,$40,$39,$3A,$40,$E9
  BYTE $3A,$BF,$A9,$3A,$AA,$A9,$3A,$AA,$A9,$3A,$AA,$A9,$35,$55,$55,$00
imageP ;pointer #61 ($0d40) 
 BYTE $3F,$FF,$40,$3A,$AA,$90,$3A,$AA,$A4,$3A,$AA,$A4,$3A,$56,$A9,$3A
 BYTE $43,$A9,$3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$43,$A9,$3A,$FE
 BYTE $A4,$3A,$AA,$A4,$3A,$AA,$90,$3A,$AA,$40,$3A,$55,$00,$3A,$40,$00
 BYTE $3A,$40,$00,$3A,$40,$00,$3A,$40,$00,$3A,$40,$00,$35,$40,$00,$00

imageR ;pointer #62 ($0d80) 
 BYTE $3F,$FF,$40,$3A,$AA,$90,$3A,$AA,$A4,$3A,$AA,$A4,$3A,$56,$A9,$3A
 BYTE $43,$A9,$3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$43,$A9,$3A,$FE
 BYTE $A4,$3A,$AA,$A4,$3A,$AA,$90,$3A,$AA,$40,$3A,$AA,$40,$3A,$AA,$90
 BYTE $3A,$7A,$A4,$3A,$4E,$A9,$3A,$43,$A9,$3A,$40,$E9,$25,$40,$15,$00
imageS  ;pointer #63 ($0dc0) 
 BYTE $00,$FF,$C0,$03,$AA,$B4,$0E,$AA,$A9,$0E,$AA,$A9,$3A,$A5,$A4,$3A
 BYTE $90,$50,$3A,$90,$00,$3A,$90,$00,$3A,$AF,$00,$0E,$AA,$C0,$0E,$AA
 BYTE $B0,$01,$AA,$A4,$00,$56,$A9,$00,$03,$A9,$00,$03,$A9,$03,$03,$A9
 BYTE $0E,$FE,$A4,$3A,$AA,$A4,$3A,$AA,$90,$0E,$AA,$40,$01,$55,$00,$00
 
*=$1100
decodetext      ;finds entries in sprite data and copies them to sprite 0 scroller image
        cmp #$60                ;CBM program studio stores letters in text with values above $60, e.g. A as $61.
        bcs decodeletter        ;it is a letter
        cmp #$20        ;space
        bne *+7
        lda #$00
        jmp decodeletter+5
        cmp #$27        ;prime
        bne *+7
        lda #39
        jmp decodeletter+5
        cmp #$28        ; (
        bne *+7
        lda #41
        jmp decodeletter+5
        cmp #$29        ; )
        bne *+7
        lda #42
        jmp decodeletter+5
        cmp #$2c        ; ,
        bne *+7
        lda #44
        jmp decodeletter+5
        cmp #$2d        ; -
        bne *+7
        lda #38
        jmp decodeletter+5
        cmp #$2e        ; .
        bne *+7
        lda #37
        jmp decodeletter+5
        cmp #$2f        ; /
        bne *+7
        lda #43
        jmp decodeletter+5
        cmp #$33        ; 3
        bne *+7
        lda #30
        jmp decodeletter+5
        cmp #$34        ; 4
        bne *+7
        lda #31
        jmp decodeletter+5
        cmp #$36        ;6
        bne *+7
        lda #33
        jmp decodeletter+5
        cmp #$3a        ; :
        bne *+7
        lda #40
        jmp decodeletter+5

decodeletter
        sec
        sbc #$60
        and #$3f        ; now letters are from 1-26
        ;multiply by 64 to get spritedata entry
        ldx #$00
        stx scrollcharaddr
        stx scrollcharaddr+1
        lsr
        ror scrollcharaddr
        lsr
        ror scrollcharaddr
        sta scrollcharaddr+1
        clc
        lda #<spritedata
        adc scrollcharaddr
        sta scrollcharaddr
        lda #>spritedata
        adc scrollcharaddr+1
        sta scrollcharaddr+1
        ldy #$3f
        ldx sgfxinverted
        beq refresh
refreshinv
        lda (scrollcharaddr),y
        eor #$ff
        sta $2400,y
        dey
        bpl refreshinv
        rts
refresh lda (scrollcharaddr),y
        sta $2400,y
        dey
        bpl refresh
        rts


*=$1c00 ;text on screen
screentext
          BYTE 32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32
          BYTE 32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32
          BYTE 32,20,8,9,19,32,4,5,13,15,32,1,16,16,12,9,5,19,32,20
          BYTE 8,5,32,18,1,18,5,12,25,32,21,19,5,4,32,32,32,32,32,32
          BYTE 32,15,16,3,15,4,5,19,32,18,12,1,32,1,14,4,32,12,1,19
          BYTE 46,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32
          BYTE 32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32
          BYTE 32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32
          BYTE 32,20,8,5,32,3,15,12,15,18,32,3,25,3,12,5,18,32,9,14
          BYTE 32,20,8,5,32,20,15,16,32,2,15,18,4,5,18,32,32,32,32,32
          BYTE 32,13,1,19,11,19,32,20,8,5,32,19,20,1,3,11,16,15,9,14
          BYTE 20,5,18,32,23,9,20,8,32,12,1,19,32,20,15,32,32,32,32,32
          BYTE 32,19,5,12,5,3,20,32,20,8,5,32,3,15,12,15,18,19,32,6
          BYTE 18,15,13,32,1,32,12,9,19,20,46,32,32,32,32,32,32,32,32,32
          BYTE 32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32
          BYTE 32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32
          BYTE 32,20,8,5,32,12,1,25,5,18,5,4,32,19,16,18,9,20,5,32
          BYTE 19,3,18,15,12,12,5,18,32,9,14,32,20,8,5,32,32,32,32,32
          BYTE 32,12,15,23,5,18,32,2,15,18,4,5,18,32,3,15,13,2,9,14
          BYTE 5,19,32,20,8,5,32,13,15,22,9,14,7,32,1,14,4,32,32,32
          BYTE 32,6,9,24,5,4,32,7,18,1,16,8,9,3,19,32,23,9,20,8
          BYTE 32,18,12,1,46,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32
          BYTE 32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32
          BYTE 32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32
          BYTE 32,3,8,5,3,11,32,15,21,20,32,20,8,5,32,1,19,13,32,6
          BYTE 9,12,5,32,6,15,18,32,13,15,18,5,32,9,14,6,15,46,32,32
          BYTE 32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32
          BYTE 32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32
          BYTE 32,3,8,1,14,7,5,32,3,15,12,15,18,19,32,23,9,20,8,32
          BYTE 6,49,44,32,6,51,44,32,6,53,44,32,1,14,4,32,6,55,46,32
          BYTE 32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32
          BYTE 32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32
          BYTE 32,9,14,22,5,18,20,32,40,5,15,18,32,35,36,6,6,41,32,6
          BYTE 9,24,5,4,32,7,6,24,32,23,9,20,8,32,6,46,32,32,32,32
          BYTE 32,9,14,22,5,18,20,32,19,12,9,4,9,14,7,32,7,6,24,32
          BYTE 23,9,20,8,32,19,46,32,32,32,32,32,32,32,32,32,32,32,32,32
          BYTE 32,16,1,21,19,5,32,19,3,18,15,12,12,5,18,32,23,9,20,8
          BYTE 32,16,46,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32
          BYTE 32,16,18,5,19,19,32,13,32,20,15,32,20,21,18,14,32,13,21,19
          BYTE 9,3,32,15,6,6,47,15,14,46,32,32,32,32,32,32,32,32,32,32
          BYTE 32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32
          BYTE 32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32
          BYTE 32,9,32,8,15,16,5,32,25,15,21,32,12,9,11,5,32,9,20,32
          BYTE 1,14,4,32,20,8,1,20,32,9,20,32,16,18,15,22,15,11,5,19
          BYTE 32,13,1,14,25,32,14,5,23,32,9,4,5,1,19,33,32,32,32,32
          BYTE 32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32
          BYTE 32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32
          BYTE 32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32
          BYTE 32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32
          BYTE 32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32
          BYTE 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,136,137,138,139
          BYTE 140,141,142,143

*=$2600
birddata        ;flying bird sprites
;spr_img0

 BYTE $00,$00,$00,$e0,$01,$24,$29,$07,$a0,$fa,$4f,$a0,$de,$76,$a0,$2a
 BYTE $ae,$a0,$3a,$ae,$a0,$2a,$ae,$90,$ea,$9a,$80,$3a,$9a,$40,$ea,$9a
 BYTE $40,$2a,$9a,$00,$ce,$ae,$00,$0e,$aa,$40,$e2,$aa,$a0,$ea,$aa,$b8
 BYTE $2a,$aa,$a9,$2a,$aa,$a5,$34,$aa,$00,$01,$68,$00,$05,$10,$00,$8c

;spr_img1

 BYTE $03,$40,$00,$07,$40,$04,$0b,$40,$f8,$db,$80,$e8,$2e,$83,$a8,$3a
 BYTE $80,$a8,$ea,$82,$a8,$3a,$80,$a8,$fa,$9e,$a4,$2a,$9a,$a0,$3a,$9a
 BYTE $90,$0a,$9a,$80,$0e,$9a,$40,$0e,$ba,$40,$e2,$aa,$a0,$ea,$aa,$b8
 BYTE $2a,$aa,$a9,$2a,$aa,$a5,$34,$aa,$00,$01,$68,$00,$05,$10,$00,$8c

;spr_img2

 BYTE $00,$00,$00,$00,$00,$10,$04,$00,$1c,$14,$03,$6c,$18,$00,$b8,$2d
 BYTE $02,$f4,$fa,$43,$a4,$de,$41,$a0,$2a,$82,$a0,$da,$92,$90,$2a,$9a
 BYTE $80,$0a,$aa,$40,$ee,$aa,$a0,$ea,$aa,$b8,$2a,$aa,$a9,$2a,$aa,$a5
 BYTE $34,$aa,$00,$01,$68,$00,$05,$10,$00,$00,$00,$00,$00,$00,$00,$8c

;spr_img3

 BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$3a,$aa,$00,$aa,$aa
 BYTE $90,$aa,$aa,$78,$ea,$aa,$a8,$ba,$aa,$a9,$2a,$aa,$95,$ba,$6a,$9c
 BYTE $29,$7a,$a4,$05,$1e,$80,$00,$00,$00,$00,$00,$00,$00,$00,$00,$8c

;spr_img4

 BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 BYTE $00,$00,$00,$00,$00,$3a,$aa,$40,$aa,$aa,$a0,$aa,$aa,$b8,$ea,$aa
 BYTE $a9,$ea,$aa,$a5,$23,$aa,$a0,$02,$a9,$a4,$02,$ab,$a4,$03,$ab,$a8
 BYTE $02,$ab,$a9,$0e,$a4,$ab,$3a,$a4,$ff,$0a,$a0,$d5,$00,$90,$04,$8c

;spr_img5

 BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 BYTE $00,$00,$00,$00,$00,$3a,$aa,$40,$aa,$aa,$a0,$aa,$aa,$b8,$ea,$aa
 BYTE $a9,$ea,$aa,$a5,$23,$a9,$a0,$02,$a9,$a4,$02,$ab,$a4,$03,$ab,$a8
 BYTE $02,$ab,$a8,$07,$a8,$6c,$03,$a8,$fd,$03,$e4,$e9,$00,$f4,$1b,$8c

;spr_img6

 BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$3a,$aa,$40,$aa,$aa
 BYTE $a0,$aa,$aa,$b8,$ea,$aa,$a9,$ea,$aa,$a5,$2a,$a9,$80,$0e,$a9,$80
 BYTE $0f,$a9,$90,$01,$a9,$90,$00,$5b,$80,$00,$5b,$80,$00,$1b,$c0,$8c

;spr_img7

 BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$40,$00,$02,$a4,$00,$0a
 BYTE $a8,$00,$39,$a8,$e0,$1b,$a7,$a0,$1b,$a6,$a0,$32,$a6,$90,$3e,$a6
 BYTE $90,$0a,$ae,$90,$ea,$aa,$a0,$ea,$aa,$b8,$2a,$aa,$a9,$2a,$aa,$a5
 BYTE $37,$aa,$50,$01,$69,$50,$05,$13,$f0,$00,$00,$30,$00,$00,$00,$8c

;spr_img8

 BYTE $00,$00,$00,$00,$00,$00,$00,$00,$a0,$0a,$43,$a4,$3c,$42,$a8,$1a
 BYTE $9e,$bc,$36,$9a,$bc,$0a,$ae,$9c,$0a,$ae,$44,$0a,$ae,$40,$0a,$9a
 BYTE $40,$0a,$9a,$00,$0e,$aa,$00,$0e,$aa,$40,$e2,$aa,$a0,$ea,$aa,$b8
 BYTE $2a,$aa,$a9,$2a,$aa,$a5,$34,$aa,$00,$01,$68,$00,$05,$10,$00,$8c



*=$2900
generateunrolledcode
        ;jmp *
        ;generate unrolled code rla
        lda #<movesprites
        sta $fd
        ldx #>movesprites
        stx $fe
        ;addresdata
        ldx #$20
        stx $02
        inx
        inx
        stx $03
        inx
        inx
        stx $04
        ldx #$02
        stx $05
        stx $06
        ;repeat 21 times (for each sprite row)
        ldx #00
        stx linecounter
        ;generate code for sprite 0
repeat  ldy #$00
        lda #$0e        ; asl abs
        sta ($fd),y
        iny
        ldx $06         ;#$02, #$05, etc.
        txa
        sta ($fd),y
        iny
        lda $04         ;#$24 of #$25
        sta ($fd),y
        iny
        lda #$2e        ; rol abs
        sta ($fd),y
        iny
        dex
        txa
        sta ($fd),y
        iny
        lda $04         ;#$24 of #$25
        sta ($fd),y
        iny
        lda #$2e        ; rol abs
        sta ($fd),y
        iny
        dex
        txa
        sta ($fd),y
        iny
        lda $04         ;#$24 of #$25
        sta ($fd),y
        iny
        ;prepare for next sprite
        clc
        lda $05
        adc #$40
        sta $05
        sta $06
        ;code for sprites 1-7
        ldx #01
        stx spritecounter
        ldx #$02        ; 3 bytes in a sprite line
sprite  ;iny
        lda #$ad        ; opcode lda abs
        sta ($fd),y
        iny
        lda $06         ;#$02 ,#$05, etc
        sta ($fd),y
        iny
        lda $03         ;#$22 of #$23
        sta ($fd),y
        iny
        lda #$2f        ;opcode rla abs
        sta ($fd),y
        iny
        lda $06         ;#$02 ,#$05, etc
        sta ($fd),y
        iny
        lda $04         ;#$24 of #$25
        sta ($fd),y
        iny
        lda #$8d        ;opcode sta abs
        sta ($fd),y
        iny
        lda $06         ;#$02 ,#$05, etc
        sta ($fd),y
        iny
        lda $02         ;#$20 of #$21
        sta ($fd),y
        iny
        ;repeat for other two bytes
        dec $06
        dex
        bpl sprite
        ;prepare for next sprite
        clc
        lda $05
        adc #$40
        sta $05
        sta $06
        bcc incrskipped
        inc $02
        inc $03
        inc $04
incrskipped
        lda #$08
        sec
        ins spritecounter       ; in Groepaz's document this is called ISC.
        bne sprite-2
        ;prepare for next line
        lda #$4c        ;opcode jmp
        sta ($fd),y
        iny
        lda #$00
        sta ($fd),y
        iny
        ldx $fe
        inx
        txa
        sta ($fd),y
        stx $fe
        ;reset adresdata
        ldx #$20
        stx $02
        inx
        inx
        stx $03
        inx
        inx
        stx $04
        clc
        lda $05
        adc #$03
        sta $05
        sta $06
        lda #21 ;last sprite line
        sec
        ins linecounter
        beq *+5
        jmp repeat
        ;replace the jmp by an rts for the last line
        dey
        dey
        dec $fe
        lda #$60        ;opcode rts
        sta ($fd),y
        rts

*=$3f00         
movesprites     ; Placeholder: all the code is generated in the subroutinee generateunrolledcode

        ;code for 1st byte of sprite 0.
        asl $2402
        rol $2401
        rol $2400
        ;then follows code with rla for the other sprites
        inx
        jmp $4000


*=$5400
; each row contains 16 colors and is selected according to background color
colorcycling
 BYTE $00,$09,$02,$08,$05,$0f,$07,$0f,$05,$08,$02,$09,$00,$00,$00,$00
 BYTE $00,$06,$0b,$04,$0c,$03,$0d,$01,$01,$0d,$03,$0c,$04,$0b,$06,$00
 BYTE $02,$08,$0a,$0f,$07,$01,$07,$0f,$0a,$08,$02,$08,$0a,$08,$02,$08
 BYTE $03,$0c,$04,$0b,$06,$00,$06,$0b,$04,$0c,$03,$0c,$04,$0c,$03,$0c
 BYTE $04,$0c,$03,$0d,$01,$0d,$03,$0c,$04,$0c,$03,$0d,$01,$0d,$03,$0c
 BYTE $05,$0c,$04,$0b,$06,$00,$06,$0b,$04,$0c,$05,$0c,$04,$0c,$05,$0c
 BYTE $06,$0b,$04,$0e,$03,$0d,$01,$0d,$03,$0e,$04,$0b,$06,$0b,$06,$0b
 BYTE $07,$0f,$0a,$08,$02,$09,$00,$09,$02,$08,$0a,$0f,$07,$0f,$07,$0f
 BYTE $08,$0c,$0f,$07,$01,$07,$0f,$0c,$08,$0c,$0f,$07,$01,$07,$0f,$0c
 BYTE $09,$02,$08,$0c,$0f,$07,$01,$07,$0f,$0c,$08,$02,$09,$02,$09,$02
 BYTE $0a,$04,$02,$09,$00,$09,$02,$04,$0a,$02,$09,$00,$09,$02,$04,$0a
 BYTE $0b,$04,$0e,$03,$0d,$01,$0d,$03,$0e,$04,$0b,$04,$0e,$04,$0b,$04
 BYTE $0c,$04,$0b,$06,$00,$06,$0b,$04,$0c,$04,$0b,$06,$00,$06,$0b,$04
 BYTE $0d,$0f,$0c,$08,$02,$09,$00,$09,$02,$08,$0c,$0f,$0d,$0f,$0d,$0f
 BYTE $0e,$0c,$04,$0b,$06,$00,$06,$0b,$04,$0c,$0e,$0c,$04,$0c,$0e,$0c
 BYTE $0f,$0c,$04,$0b,$06,$00,$06,$0b,$04,$0c,$0f,$0c,$04,$0c,$0f,$0c


*=$5500 ;scrolltekst
scrolltext
 TEXT "  i always found the 'illegal' opcodes (henceforth called uocs :-)) very interesting. "
 TEXT "the 'first unofficial csdb uoc coding challenge' got me thinking about potential applications. "
 TEXT "thanks to monte carlos for organizing this compo. after studying groepaz's excellent document, "
 TEXT "i came up with ideas for the rarely used opcodes rla and las. but since this message is already "
 TEXT "quite long, i suggest you peek into the listing for explanations. thanks to v3to for his color fading doc. "
 TEXT "sprites are from kofler.dot.at/c64/. music by tonal kaos. sorry for this boring text. maybe someone should organize a funny scrolltext compo. "
 TEXT "bye for now.             "
 BYTE 0,0

*=$5900 ;LAS mask (Senoritas)
 BYTE 15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15
 BYTE 15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15
 BYTE 15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15
 BYTE 15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15
 BYTE 15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15
 BYTE 15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15
 BYTE 15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15
 BYTE 15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15
 BYTE 15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15
 BYTE 15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15
 BYTE 15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15
 BYTE 15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15
 BYTE 15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15
 BYTE 15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15
 BYTE 15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15
 BYTE 15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15

*=$5a00 ;lowbytes of color cycler color schemes
colorscheme
 BYTE 0,16,32,48,64,80,96,112,128,144,160,176,192,208,224,240

*=$5b00 ;sprite data
spritedata
;space
 BYTE   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
 BYTE   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
 BYTE   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
 BYTE   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

; A
 BYTE $00,$0C,$00,$00,$39,$00,$00,$EA,$40,$00,$EA,$40,$03,$EA,$40,$03
 BYTE $AA,$90,$03,$AA,$90,$03,$AA,$90,$03,$AA,$90,$0E,$A6,$90,$0E,$93
 BYTE $A4,$0E,$93,$A4,$0E,$93,$A4,$0E,$93,$A4,$3A,$93,$A4,$3A,$AF,$A9
 BYTE $3A,$AA,$A9,$3A,$AA,$A9,$3A,$55,$E9,$3A,$40,$E9,$25,$40,$95,$00

; B
 BYTE $FF,$FC,$00,$EA,$A9,$00,$EA,$AA,$40,$EA,$AA,$40,$E9,$5E,$90,$E9
 BYTE $03,$90,$E9,$03,$90,$E9,$0E,$90,$EA,$FA,$40,$EA,$AA,$40,$EA,$A9
 BYTE $00,$EA,$AA,$40,$E9,$5A,$40,$E9,$0E,$90,$E9,$03,$90,$E9,$03,$90
 BYTE $E9,$0E,$90,$EA,$FA,$40,$EA,$AA,$40,$EA,$A9,$00,$95,$54,$00,$00

; C
 BYTE $0F,$FF,$F0,$0E,$AA,$A4,$3A,$AA,$A9,$3A,$AA,$A9,$3A,$95,$69,$3A
 BYTE $40,$39,$3A,$40,$39,$3A,$40,$25,$3A,$40,$00,$3A,$40,$00,$3A,$40
 BYTE $00,$3A,$40,$00,$3A,$40,$00,$3A,$40,$3E,$3A,$40,$39,$3A,$40,$39
 BYTE $3A,$BF,$F9,$0E,$AA,$A9,$0E,$AA,$A9,$03,$AA,$A4,$00,$55,$50,$00

; D
 BYTE $FF,$00,$00,$EA,$FC,$00,$EA,$AB,$00,$EA,$AA,$40,$EA,$AA,$90,$E9
 BYTE $6A,$90,$E9,$1A,$A4,$E9,$07,$A4,$E9,$03,$A4,$E9,$03,$A4,$E9,$03
 BYTE $A4,$E9,$03,$A4,$E9,$03,$A4,$E9,$03,$A4,$E9,$0E,$A4,$E9,$3A,$A4
 BYTE $EB,$EA,$90,$EA,$AA,$90,$EA,$AA,$40,$EA,$95,$00,$E9,$40,$00,$00

; E
 BYTE $FF,$FF,$F8,$EA,$AA,$A4,$EA,$AA,$A4,$EA,$AA,$A4,$EA,$AA,$A4,$E9
 BYTE $55,$54,$E9,$00,$00,$E9,$00,$00,$EB,$F8,$00,$EA,$A4,$00,$EA,$A4
 BYTE $00,$EA,$A4,$00,$E9,$54,$00,$E9,$00,$00,$E9,$00,$00,$EB,$FF,$F8
 BYTE $EA,$AA,$A4,$EA,$AA,$A4,$EA,$AA,$A4,$EA,$AA,$A4,$95,$55,$54,$00

; F
 BYTE $3F,$FF,$FE,$3A,$AA,$A9,$3A,$AA,$A9,$3A,$AA,$A9,$3A,$AA,$A9,$3A
 BYTE $55,$55,$3A,$40,$00,$3A,$40,$00,$3A,$FE,$00,$3A,$A9,$00,$3A,$A9
 BYTE $00,$3A,$A9,$00,$3A,$55,$00,$3A,$40,$00,$3A,$40,$00,$3A,$40,$00
 BYTE $3A,$40,$00,$3A,$40,$00,$3A,$40,$00,$3A,$40,$00,$25,$40,$00,$00

; G
 BYTE $00,$FF,$C0,$03,$AA,$B0,$0E,$AA,$94,$0E,$AA,$A4,$3A,$96,$A9,$3A
 BYTE $43,$A9,$3A,$40,$D5,$3A,$40,$00,$3A,$40,$00,$3A,$4F,$FE,$3A,$4E
 BYTE $A9,$3A,$4E,$A9,$3A,$4E,$A9,$3A,$49,$A9,$3A,$40,$E9,$3A,$C3,$A9
 BYTE $3A,$BE,$A9,$0E,$AA,$A4,$0E,$AA,$A4,$03,$AA,$90,$00,$55,$40,$00

; H
 BYTE $3F,$80,$FE,$3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A
 BYTE $40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$BF,$E9,$3A,$AA,$A9,$3A,$AA
 BYTE $A9,$3A,$AA,$A9,$3A,$55,$A9,$3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9
 BYTE $3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9,$25,$40,$95,$00

; I
 BYTE $3F,$FF,$F8,$3A,$AA,$A4,$3A,$AA,$A4,$3A,$AA,$A4,$3A,$AA,$A4,$25
 BYTE $A9,$54,$00,$E9,$00,$00,$E9,$00,$00,$E9,$00,$00,$E9,$00,$00,$E9
 BYTE $00,$00,$E9,$00,$00,$E9,$00,$00,$E9,$00,$00,$E9,$00,$3F,$EA,$F8
 BYTE $3A,$AA,$A4,$3A,$AA,$A4,$3A,$AA,$A4,$3A,$AA,$A4,$25,$55,$54,$00

; J
 BYTE $3F,$FF,$F8,$3A,$AA,$A4,$3A,$AA,$A4,$3A,$AA,$A4,$3A,$AA,$A4,$25
 BYTE $56,$A4,$00,$03,$A4,$00,$03,$A4,$00,$03,$A4,$00,$03,$A4,$00,$03
 BYTE $A4,$00,$03,$A4,$3F,$83,$A4,$3A,$43,$A4,$3A,$43,$A4,$3A,$43,$A4
 BYTE $3A,$BE,$A4,$0E,$AA,$90,$0E,$AA,$90,$03,$AA,$40,$00,$55,$00,$00

; K
 BYTE $3F,$80,$FE,$3A,$43,$A9,$3A,$43,$A9,$3A,$4E,$A9,$3A,$4E,$A4,$3A
 BYTE $7A,$A4,$3A,$AA,$90,$3A,$AA,$90,$3A,$AA,$40,$3A,$A9,$00,$3A,$A9
 BYTE $00,$3A,$AA,$40,$3A,$AA,$40,$3A,$6A,$90,$3A,$7A,$90,$3A,$7A,$A4
 BYTE $3A,$4E,$A4,$3A,$4E,$A9,$3A,$43,$A9,$3A,$43,$A9,$25,$40,$D5,$00

; L
 BYTE $3F,$80,$00,$3A,$40,$00,$3A,$40,$00,$3A,$40,$00,$3A,$40,$00,$3A
 BYTE $40,$00,$3A,$40,$00,$3A,$40,$00,$3A,$40,$00,$3A,$40,$00,$3A,$40
 BYTE $00,$3A,$40,$00,$3A,$40,$00,$3A,$40,$0E,$3A,$40,$39,$3A,$40,$E9
 BYTE $3A,$BF,$A9,$3A,$AA,$A9,$3A,$AA,$A9,$3A,$AA,$A9,$25,$55,$55,$00

; M
 BYTE $3F,$80,$FE,$3A,$40,$E9,$3A,$93,$A9,$3A,$93,$A9,$3A,$AE,$A9,$3A
 BYTE $AA,$A9,$3A,$AA,$A9,$3A,$AA,$A9,$3A,$EA,$A9,$3A,$79,$E9,$3A,$55
 BYTE $E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9
 BYTE $3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9,$25,$40,$95,$00

; N
 BYTE $3F,$40,$FE,$3A,$90,$E9,$3A,$90,$E9,$3A,$90,$E9,$3A,$A4,$E9,$3A
 BYTE $A4,$E9,$3A,$A4,$E9,$3A,$A9,$E9,$3A,$A9,$E9,$3A,$69,$E9,$3A,$6A
 BYTE $E9,$3A,$7A,$A9,$3A,$4E,$A9,$3A,$4E,$A9,$3A,$4E,$A9,$3A,$43,$A9
 BYTE $3A,$43,$A9,$3A,$43,$A9,$3A,$40,$E9,$3A,$40,$E9,$25,$40,$95,$00

; O
 BYTE $00,$FF,$40,$03,$AA,$90,$03,$AA,$90,$0E,$AA,$A4,$0E,$AE,$A4,$0E
 BYTE $93,$A4,$3A,$93,$A9,$3A,$93,$A9,$3A,$40,$E9,$3A,$40,$E9,$3A,$40
 BYTE $E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$93,$A9,$3A,$93,$A9,$0E,$93,$A4
 BYTE $0E,$AE,$A4,$0E,$AA,$A4,$03,$AA,$90,$03,$AA,$90,$00,$55,$40,$00

; P
 BYTE $3F,$FF,$40,$3A,$AA,$90,$3A,$AA,$A4,$3A,$AA,$A4,$3A,$56,$A9,$3A
 BYTE $43,$A9,$3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$43,$A9,$3A,$FE
 BYTE $A4,$3A,$AA,$A4,$3A,$AA,$90,$3A,$AA,$40,$3A,$55,$00,$3A,$40,$00
 BYTE $3A,$40,$00,$3A,$40,$00,$3A,$40,$00,$3A,$40,$00,$25,$40,$00,$00

; Q
 BYTE $00,$FF,$40,$03,$AA,$90,$03,$AA,$90,$0E,$AA,$A4,$0E,$AE,$A4,$0E
 BYTE $93,$A4,$3A,$93,$A9,$3A,$93,$A9,$3A,$40,$E9,$3A,$40,$E9,$3A,$40
 BYTE $E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$93,$69,$3A,$93,$99,$0E,$9F,$A4
 BYTE $0E,$AE,$A9,$0E,$AB,$A9,$03,$AB,$A9,$03,$AA,$E9,$00,$55,$15,$00

; R
 BYTE $3F,$FF,$40,$3A,$AA,$90,$3A,$AA,$A4,$3A,$AA,$A4,$3A,$56,$A9,$3A
 BYTE $43,$A9,$3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$43,$A9,$3A,$FE
 BYTE $A4,$3A,$AA,$A4,$3A,$AA,$90,$3A,$AA,$40,$3A,$AA,$40,$3A,$AA,$90
 BYTE $3A,$7A,$A4,$3A,$4E,$A9,$3A,$43,$A9,$3A,$40,$E9,$25,$40,$15,$00

; S
 BYTE $00,$FF,$C0,$03,$AA,$B4,$0E,$AA,$A9,$0E,$AA,$A9,$3A,$A5,$A4,$3A
 BYTE $90,$50,$3A,$90,$00,$3A,$90,$00,$3A,$AF,$00,$0E,$AA,$C0,$0E,$AA
 BYTE $B0,$01,$AA,$A4,$00,$56,$A9,$00,$03,$A9,$00,$03,$A9,$03,$03,$A9
 BYTE $0E,$FE,$A4,$3A,$AA,$A4,$3A,$AA,$90,$0E,$AA,$40,$01,$55,$00,$00

; T
 BYTE $3F,$FF,$F8,$3A,$AA,$A4,$3A,$AA,$A4,$3A,$AA,$A4,$3A,$AA,$A4,$3A
 BYTE $AA,$A4,$25,$E9,$54,$00,$E9,$00,$00,$E9,$00,$00,$E9,$00,$00,$E9
 BYTE $00,$00,$E9,$00,$00,$E9,$00,$00,$E9,$00,$00,$E9,$00,$00,$E9,$00
 BYTE $00,$E9,$00,$00,$E9,$00,$00,$E9,$00,$00,$E9,$00,$00,$95,$00,$00

; U
 BYTE $3F,$80,$FE,$3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A
 BYTE $40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$40
 BYTE $E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$BF,$A9
 BYTE $3A,$AA,$A9,$3A,$AA,$A9,$3A,$AA,$A9,$0E,$AA,$A4,$01,$55,$50,$00

; V
 BYTE $3F,$80,$FE,$3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$93,$A9,$0E
 BYTE $93,$A4,$0E,$93,$A4,$0E,$93,$A4,$0E,$A6,$A4,$0E,$A6,$A4,$03,$AA
 BYTE $90,$03,$AA,$90,$03,$AA,$90,$03,$AA,$90,$03,$AA,$90,$00,$EA,$40
 BYTE $00,$EA,$40,$00,$EA,$40,$00,$39,$00,$00,$39,$00,$00,$04,$00,$00

; W
 BYTE $3F,$80,$FE,$3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A
 BYTE $40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$40,$E9,$3A,$4C
 BYTE $E9,$3A,$79,$E9,$3A,$6A,$E9,$3A,$AA,$A9,$3A,$AA,$A9,$3A,$AA,$A9
 BYTE $3A,$A6,$A9,$3A,$93,$A9,$3A,$93,$A9,$3A,$40,$E9,$25,$40,$95,$00

; X
 BYTE $3E,$00,$3E,$3A,$40,$E9,$3A,$40,$E9,$3A,$93,$A9,$3A,$93,$A9,$3A
 BYTE $AE,$A9,$0E,$AA,$A4,$0E,$AA,$A4,$03,$AA,$90,$03,$AA,$90,$00,$AA
 BYTE $40,$03,$AA,$90,$03,$AA,$90,$0E,$AA,$A4,$0E,$AA,$A4,$3A,$AE,$A9
 BYTE $3A,$93,$A9,$3A,$93,$A9,$3A,$40,$E9,$3A,$40,$E9,$25,$00,$15,$00

;Y
 BYTE 63,128,254,58,64,233,58,64,233,58,64,233,58,64,233,58
 BYTE 64,233,58,191,233,58,170,169,58,170,169,58,170,169,58,170
 BYTE 169,37,85,169,0,0,233,0,0,233,0,0,233,63,255,233
 BYTE 58,170,169,58,170,169,58,170,169,58,170,169,37,85,85,0
;Z
 BYTE 63,255,254,58,170,169,58,170,169,58,170,169,58,170,169,37
 BYTE 90,169,0,10,164,0,58,164,0,42,144,0,234,144,0,234
 BYTE 64,3,170,64,3,169,0,14,169,0,14,164,0,58,171,254
 BYTE 58,170,169,58,170,169,58,170,169,58,170,169,37,85,85,0

;0-9
;zero
 BYTE 3,255,192,14,170,144,58,170,164,58,170,164,234,150,169,234
 BYTE 67,169,234,67,169,234,67,169,234,67,169,234,67,169,234,67
 BYTE 169,234,67,169,234,67,169,234,67,169,234,67,169,234,67,169
 BYTE 234,190,169,58,170,164,58,170,164,14,170,144,1,85,64,0

;one
 BYTE 3,254,0,3,169,0,3,169,0,3,169,0,3,169,0,3
 BYTE 169,0,3,169,0,3,169,0,3,169,0,3,169,0,3,169
 BYTE 0,3,169,0,3,169,0,3,169,0,3,169,0,3,169,0
 BYTE 3,169,0,3,169,0,3,169,0,3,169,0,2,85,0,0

;two
 BYTE 3,255,192,14,170,144,58,170,164,58,170,164,234,150,169,234
 BYTE 67,169,234,67,169,234,67,169,149,67,164,0,14,164,0,58
 BYTE 144,0,234,64,3,169,0,14,164,0,58,144,0,234,64,0
 BYTE 234,255,254,234,170,169,234,170,169,234,170,169,149,85,85,0

;three
 BYTE 3,255,192,14,170,144,58,170,164,58,170,164,234,150,169,234
 BYTE 67,169,234,67,169,234,67,169,149,67,164,0,14,164,0,58
 BYTE 144,0,14,164,255,131,164,234,67,169,234,67,169,234,67,169
 BYTE 234,170,169,58,170,164,58,170,164,14,170,144,1,85,64,0

;four
 BYTE 3,248,0,14,164,0,14,164,0,14,164,0,58,144,0,58
 BYTE 144,0,58,144,0,58,159,224,234,122,144,234,122,144,234,122
 BYTE 144,234,122,164,234,170,169,234,170,169,234,170,169,149,106,164
 BYTE 0,58,144,0,58,144,0,58,144,0,58,144,0,37,80,0

;five
 BYTE 255,255,254,234,170,169,234,170,169,234,170,169,234,85,85,234
 BYTE 64,0,234,255,192,234,170,144,234,170,164,234,170,164,149,86
 BYTE 169,0,3,169,0,3,169,255,131,169,234,67,169,234,67,169
 BYTE 234,190,169,58,170,164,58,170,164,14,170,144,1,85,64,0

;six
 BYTE 0,15,224,0,58,144,0,58,144,0,234,64,0,234,64,3
 BYTE 169,0,3,169,0,14,170,64,14,170,144,58,170,164,58,150
 BYTE 164,234,67,169,234,67,169,234,67,169,234,67,169,234,67,169
 BYTE 234,190,169,58,170,164,58,170,164,14,170,144,1,85,64,0

;seven
 BYTE 255,255,254,234,170,169,234,170,169,234,170,169,149,90,169,0
 BYTE 14,164,0,14,164,0,58,144,0,58,144,0,58,144,0,234
 BYTE 64,0,234,64,0,234,64,0,234,64,0,234,64,0,234,64
 BYTE 0,234,64,0,234,64,0,234,64,0,234,64,0,149,64,0

;eight
 BYTE 3,255,192,14,170,144,58,170,164,58,170,164,234,150,169,234
 BYTE 67,169,234,67,169,234,67,169,58,190,164,58,170,164,14,170
 BYTE 144,58,170,164,58,150,164,234,67,169,234,67,169,234,67,169
 BYTE 234,190,169,58,170,164,58,170,164,14,170,144,1,85,64,0

;nine
 BYTE 3,255,192,14,170,144,58,170,164,58,170,164,234,150,169,234
 BYTE 67,169,234,67,169,234,67,169,234,67,169,234,67,169,58,190
 BYTE 164,58,170,164,14,170,144,3,170,144,0,234,64,0,234,64
 BYTE 3,169,0,3,169,0,14,164,0,14,164,0,9,80,0,0

;dot .
 BYTE   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
 BYTE   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
 BYTE   0,0,240,0,15,164,0,14,169,0,14,169,0,14,169,0
 BYTE   2,169,0,3,164,0,0,80,0,0,0,0,0,0,0,0
;hyphen -
 BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 BYTE $00,$00,$00,$00,$00,$00,$00,$00,$FF,$FF,$F8,$EA,$AA,$A4,$EA,$AA
 BYTE $A4,$EA,$AA,$A4,$95,$55,$54,$00,$00,$00,$00,$00,$00,$00,$00,$00
 BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00;
;prime '
 BYTE $00,$00,$00,$00,$BE,$00,$00,$E9,$00,$00,$E9,$00,$00,$E9,$00,$00
 BYTE $E9,$00,$00,$E9,$00,$00,$E9,$00,$00,$28,$00,$00,$14,$00,$00,$00
 BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
;colon :
 BYTE $00,$00,$00,$00,$3C,$00,$03,$E9,$00,$03,$AA,$40,$03,$AA,$40,$03
 BYTE $AA,$40,$00,$AA,$40,$00,$E9,$00,$00,$14,$00,$00,$00,$00,$00,$00
 BYTE $00,$00,$00,$00,$00,$3C,$00,$03,$E9,$00,$03,$AA,$40,$03,$AA,$40
 BYTE $03,$AA,$40,$00,$AA,$40,$00,$E9,$00,$00,$14,$00,$00,$00,$00,$00

;left (
 BYTE $00,$00,$94,$00,$00,$e4,$00,$00,$e4,$00,$03,$a4,$00,$03,$a4,$00
 BYTE $0f,$90,$00,$0e,$90,$00,$0e,$80,$00,$0e,$40,$00,$3a,$40,$00,$3a
 BYTE $40,$00,$3a,$40,$00,$0e,$40,$00,$0e,$80,$00,$0e,$90,$00,$0f,$90
 BYTE $00,$03,$a4,$00,$03,$a4,$00,$00,$e4,$00,$00,$e4,$00,$00,$94,$00

;right )
 BYTE $3e,$00,$00,$39,$00,$00,$39,$00,$00,$3a,$40,$00,$3a,$40,$00,$0e
 BYTE $50,$00,$0e,$90,$00,$02,$90,$00,$03,$90,$00,$03,$a4,$00,$03,$a4
 BYTE $00,$03,$a4,$00,$03,$90,$00,$02,$90,$00,$0e,$90,$00,$0e,$50,$00
 BYTE $3a,$40,$00,$3a,$40,$00,$39,$00,$00,$39,$00,$00,$25,$00,$00,$00

;slash /
 BYTE $00,$0F,$E0,$00,$0E,$90,$00,$0E,$90,$00,$3A,$40,$00,$3A,$40,$00
 BYTE $3A,$40,$00,$E9,$00,$00,$E9,$00,$00,$E9,$00,$03,$A4,$00,$03,$A4
 BYTE $00,$03,$A4,$00,$0E,$90,$00,$0E,$90,$00,$0E,$90,$00,$3A,$40,$00
 BYTE $3A,$40,$00,$3A,$40,$00,$E9,$00,$00,$E9,$00,$00,$95,$00,$00,$00

;comma ,
 BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
 BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$02,$50,$00,$03,$90,$00
 BYTE $03,$90,$00,$0e,$90,$00,$0e,$90,$00,$3a,$40,$00,$39,$00,$00,$00

